/**
 * 3d secure authentication using Pci Bookings Orchestra service
 */
var ts3DSecure = {
    /**
     * The session token as generated by ePayTools
     */
    sessionToken: null,

    /**
     * handle for a Polling background task
     */
    backgroundInterval: null,

    /**
     * An indication for the polling task to stop
     */
    stopPollingSemaphore: false,

    /**
     * iFrame for challenge/fingerprint
     */
    iFrame : null,

    /**
     * The callback function to be invoked upon completion of the 3DS authentication
     */
    callback : null,

    /**
     * Base url for Api
     */
    apiBaseUrl : "https://api.epaytools.com",

    /**
     * Main function to perform a 3d secure authentication session
     * 
     * @param {string} sessionToken - token obtained from the Orchestra service
     * @param {string} iFrameElementId - Id of element in which a challenge iFrame is to be created
     * @param {Object} cardData - The card details
     * @param callback - the callback function to be invoked upon completion
     */

    perform3ds: function(sessionToken, iFrameElementId, cardData, callback) {
        if (!iFrameElementId || !cardData  || !sessionToken || !callback) {
            console.error("mainContentId || iFrameElementId or cardData or sessionToken or callback are empty; quitting");
            return;
        }

        this.iFrame = document.getElementById(iFrameElementId);
        if (!this.iFrame) {
            console.error(iFrameElementId + " not accessible");
            return;
        }

        // store callback function + sessionToken
        this.callback = callback;
        this.sessionToken = sessionToken;
        // Init 3DS session 
        ts3DSecure.init3DSecAuthSession(cardData);
    },

    /**
     * init 3DS Authentication Session
     * @param {*} modelData - containing card info
     */
    init3DSecAuthSession: function (modelData) {
        // Extend card model with browser/screen/window properties 
        modelData.windowWidth = self.innerWidth;
        modelData.windowHeight = self.innerHeight;
        modelData.timeZoneOffsetMinutes = new Date().getTimezoneOffset();
        modelData.colorDepth = screen.colorDepth;
        modelData.browserScreenWidth = screen.availWidth;
        modelData.browserScreenHeight = screen.availHeight;
        modelData.browserJavaEnabled = navigator.BrowserJavaEnabled;
        var data = JSON.stringify(modelData);
        console.log(data);
      
        const xhr = new XMLHttpRequest();

        xhr.onload = function () {
            // Request finished. Do processing here.
            var response = JSON.parse(this.responseText);
            if (xhr.status < 300)
            // Success
                ts3DSecure.handleInitSessionResponse(response);
            else
            // Failure
                ts3DSecure.handleError(this.responseText,"init3DSecAuthSession Error");
        };
          
        xhr.ontimeout = function (e) {
            // XMLHttpRequest timed out
            ts3DSecure.handleError("Http post request timed out","init3DSecAuthSession")        ;
        };

        xhr.open("POST", this.apiBaseUrl + "/threeDsSessions/init", true);
        xhr.timeout = 20000; // 20 sec. (time in milliseconds)
        xhr.setRequestHeader("Content-type", "application/json");
        xhr.setRequestHeader("Authorization", "Bearer " + this.sessionToken);
        xhr.send(data);
    },

    /**
     * Handle response for init session
     * 
     * @param {Object} message - message received from server
     */
    handleInitSessionResponse: function(message)
    {
        console.log(' init: Recieved ' + message.clientMessage.messageType + ' message from server');
        switch (message.clientMessage.messageType.toLowerCase()) {
            case 'technicalproblem':
            case 'rejected':
            case 'authenticated':
            case 'cardnotenrolled':
                // no more action required;
                this.callback(message);
                break;

            case 'challengerequired':
                // Store new token
                ts3DSecure.sessionToken = message.token;
                this.performChallenge(message.clientMessage.ascUri);         
                break;

            case 'fingerprintrequired':
                // Store new token
                ts3DSecure.sessionToken = message.token;

                // Set timer on overall Fingerprint flow
                this.startPollResults("fingerPrintStatus?isNotified=false");


                ts3DSecure.backgroundInterval = setTimeout(()=>{
                    ts3DSecure.handleError("Timeout on receiving  response after creating a fingerprint iFrame","fingerprintrequired")
                    // try obtaining fingerprint status even if notification has not been received
                    ts3DSecure.stopPollingResults();
                }, 20000);

                console.log("Creating a fingerprint iFrame");
                var ifrm = document.createElement("iframe");
                ifrm.style.display = "none";
                document.body.appendChild(ifrm);

                // inject form into iFrame, then perform a submit operation
                var body =
                "<body onload='document.getElementById(\"frmRedirect\").submit();'>" +
                "<form method='post' id='frmRedirect' action='" + message.clientMessage.fingerPrintUri + "'><input type='hidden' name='threeDSMethodData' value='" + message.clientMessage.fingerPrintData + "' /><input type='submit'/></form>" +
                "</body>";
 
                var doc = ifrm.contentWindow.document;
                doc.open();
                doc.write(body);
                doc.close();
       
                break;
    
            default:
                console.error('Received unknown message - ' + message.MessageType);
        }
    },

    /**
     * Perform challenge. Can be called either after session initiation or after a fingerprint test
     * @param {string} ascUri - the URI to use for iFrame src tag.
     */
    performChallenge: function (ascUri) {   
        var challengeTimeout = setTimeout(()=>{
            ts3DSecure.handleError("Timeout on receiving response after creating a challenge iFrame","challengerequired")
        }, 600000 /* 10 min. */);

        // Start polling challenge results
        this.startPollResults("challengeStatus");

        // Open iframe with source to challenge over the entire existing window
        console.log("Creating a challenge iFrame");

        this.iFrame.style.display = "inline";
        this.iFrame.setAttribute("src", ascUri);
    },

    /**
     * Start polling a 3DS result
     * @param {string} path - rightmost part of the path
     */
    startPollResults: function (path) {
        ts3DSecure.stopPollingSemaphore = false;
        ts3DSecure.backgroundInterval = setTimeout(function (path) {
             // perform a get operation
            console.log('Trying to obtain ' + path + ' status');
            // This ajax call will retrieve 3DS authentication status
            // Send a request to get results
            const xhr = new XMLHttpRequest();
            xhr.onload = function() {
                if (xhr.status < 300) {

                    // No further action needed since the finger print status will be received through the socket
                    console.log(path + ' inquery returned successfuly');
                    var message = JSON.parse(this.responseText);

                    // if result not available yet, proceed by resetting a deferred function.
                    if (message.MessageType === "NoResponse") {
                        if (!ts3DSecure.stopPollingSemaphore) {
                            console.log('Response not ready yet; Keep on polling');
                            ts3DSecure.startPollResults(path);
                        } else {
                            console.log('Response not ready yet; Stop polling was signalled => polling stopped');
                            ts3DSecure.handleError("Timeout while Polling results","Polling timeout");
                        }
                        return;
                    }
                    // if result available, stop polling (do not set the timer again) and handle the message
                    console.log('Response received; Stop polling and handle message');
                    
                    if (message.messageType.toLowerCase() == 'challengerequired')
                        // challenge required
                        ts3DSecure.performChallenge(message.ascUri);
                    else
                        // final status
                        ts3DSecure.handleServerResultMessage(message);
                }
                else {
                    ts3DSecure.handleError(this.responseText,"Polling error");
                }
            };

            xhr.ontimeout = function (e) {
                // XMLHttpRequest timed out.
                ts3DSecure.handleError("Http get request timed out","Polling");
            };

            xhr.open("GET", ts3DSecure.apiBaseUrl + "/threeDsSessions/" + path, true);
            xhr.timeout = 20000; // 20 sec
            xhr.setRequestHeader("Authorization", "Bearer " + ts3DSecure.sessionToken);
            xhr.send();
        }, 2000, path);
    },

    /**
     * Stop polling for results
     */
    stopPollingResults: function () {
        console.log('Signalling a Stop polling and stopping a possible pending poll');
        ts3DSecure.stopPollingSemaphore = true;
        clearTimeout(ts3DSecure.backgroundInterval);
    },

    /**
     * Handle Server result Message
     * 
     * @param {*} message - message to be sent back to caller
     */
    handleServerResultMessage: function (message) {
        console.log('3DS: Received ' + message.messageType + ' message from server');
        this.callback(message);
    },

    /**
     * Error reporting
     * @param {string} reason
     * @param {string} ref
     */
    handleError: function name(reason, ref) {
        console.error(ref + ': ' + reason);
        var errMessage = {
            messageType :'TechnicalProblem',
            reason: reason,
            reference: ref   
        };
        this.callback(errMessage);
    }
};